@@ -372,6 +372,9 @@ ebbm_update_cache_cb (gpointer data)
 	if (book_view)
 		e_data_book_view_notify_complete (book_view, NULL);
 
+	/* indicate the thread is not running */
+	g_cancellable_cancel (priv->update_cache);
+
 	return NULL;
 }
 
@@ -1058,6 +1061,8 @@ ebbm_operation_cb (OperationBase *op, gboolean cancelled, EBookBackend *backend)
 	} break;
 	}
 
+	if (op->book)
+		g_object_unref (op->book);
 	g_free (op);
 }
 
@@ -1075,6 +1080,9 @@ base_op_abstract (EBookBackend *backend, EDataBook *book, guint32 opid, Operatio
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	if (book)
+		g_object_ref (book);
+
 	op = g_new0 (OperationBase, 1);
 	op->ot = ot;
 	op->book = book;
@@ -1097,6 +1105,9 @@ str_op_abstract (EBookBackend *backend, EDataBook *book, guint32 opid, const gch
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	if (book)
+		g_object_ref (book);
+
 	op = g_new0 (OperationStr, 1);
 	op->base.ot = ot;
 	op->base.book = book;
@@ -1182,6 +1193,9 @@ ebbm_op_remove_contacts (EBookBackend *backend, EDataBook *book, guint32 opid, G
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	if (book)
+		g_object_ref (book);
+
 	op = g_new0 (OperationIDList, 1);
 	op->base.ot = OP_REMOVE_CONTACTS;
 	op->base.book = book;
@@ -1261,6 +1275,9 @@ ebbm_op_authenticate_user (EBookBackend *backend, EDataBook *book, guint32 opid,
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	if (book)
+		g_object_ref (book);
+
 	op = g_new0 (OperationAuthenticateUser, 1);
 	op->base.ot = OP_AUTHENTICATE_USER;
 	op->base.book = book;
@@ -1287,6 +1304,7 @@ e_book_backend_mapi_init (EBookBackendMAPI *ebma)
 	priv->conn_lock = g_mutex_new ();
 
 	priv->update_cache = g_cancellable_new ();
+	priv->update_cache_thread = NULL;
 }
 
 static void
@@ -1932,7 +1950,7 @@ mapi_error_to_edb_error (GError **perror, const GError *mapi_error, EDataBookSta
 	if (context)
 		err_msg = g_strconcat (context, mapi_error ? ": " : NULL, mapi_error ? mapi_error->message : NULL, NULL);
 	else if (!mapi_error)
-		err_msg = g_strdup (_("Uknown error"));
+		err_msg = g_strdup (_("Unknown error"));
 
 	g_propagate_error (perror, e_data_book_create_error (code, err_msg ? err_msg : mapi_error->message));
 
