@@ -28,8 +28,6 @@
 #include <libmapi/libmapi.h>
 #include <exchange-mapi-connection.h>
 
-#define PATH_FOLDER ".evolution/mail/mapi"
-
 /* Standard GObject macros */
 #define CAMEL_TYPE_MAPI_FOLDER \
 	(camel_mapi_folder_get_type ())
@@ -55,61 +53,6 @@
 
 G_BEGIN_DECLS
 
-typedef enum  {
-	MAPI_ITEM_TYPE_MAIL=1,
-	MAPI_ITEM_TYPE_APPOINTMENT,
-	MAPI_ITEM_TYPE_CONTACT,
-	MAPI_ITEM_TYPE_JOURNAL,
-	MAPI_ITEM_TYPE_TASK
-} MapiItemType;
-
-typedef enum  {
-	PART_TYPE_PLAIN_TEXT=1,
-	PART_TYPE_TEXT_HTML
-} MapiItemPartType;
-
-typedef struct {
-	gchar *subject;
-	gchar *from;
-	gchar *from_email;
-	gchar *from_type;
-
-	gchar *references;
-	gchar *message_id;
-	gchar *in_reply_to;
-	/*TODO : Obsolete this. Moved to recipient list*/
-	gchar *to;
-	gchar *cc;
-	gchar *bcc;
-
-	gint flags;
-	glong size;
-	time_t recieved_time;
-	time_t send_time;
-	guint cpid; /* codepage id */
-	gchar *transport_headers;
-} MapiItemHeader;
-
-typedef struct {
-	GSList *body_parts;
-} MapiItemMessage;
-
-typedef struct  {
-	mapi_id_t fid;
-	mapi_id_t mid;
-
-	MapiItemHeader header;
-	MapiItemMessage msg;
-
-	gboolean is_cal;
-
-	GSList *recipients;
-	GSList *attachments;
-	GSList *generic_streams;
-}MapiItem;
-
-void mapi_item_free (MapiItem *item);
-
 typedef struct _CamelMapiFolder CamelMapiFolder;
 typedef struct _CamelMapiFolderClass CamelMapiFolderClass;
 typedef struct _CamelMapiFolderPrivate CamelMapiFolderPrivate;
@@ -145,12 +88,12 @@ GType camel_mapi_folder_get_type (void);
 
 /* implemented */
 CamelFolder *
-camel_mapi_folder_new(CamelStore *store, const gchar *folder_name, const gchar *folder_dir, guint32 flags, CamelException *ex);
+camel_mapi_folder_new(CamelStore *store, const gchar *folder_name, const gchar *folder_dir, guint32 flags, GError **error);
 
-void mapi_update_summary ( CamelFolder *folder, GList *item_list,CamelException *ex);
-void mapi_refresh_folder(CamelFolder *folder, CamelException *ex);
-gboolean camel_mapi_folder_fetch_summary (CamelStore *store, const mapi_id_t fid, struct mapi_SRestriction *res,
-					  struct SSortOrderSet *sort, fetch_items_data *fetch_data, guint32 options);
+void mapi_update_summary ( CamelFolder *folder, GList *item_list,GError **error);
+gboolean mapi_refresh_folder(CamelFolder *folder, GError **error);
+gboolean camel_mapi_folder_fetch_summary (CamelStore *store, CamelFolder *folder, const mapi_id_t fid, struct mapi_SRestriction *res,
+					  struct SSortOrderSet *sort, fetch_items_data *fetch_data, guint32 options, GError **mapi_error);
 
 G_END_DECLS
 
