@@ -312,7 +312,7 @@ exchange_mapi_cal_util_fetch_organizer (ECalComponent *comp, GSList **recip_list
 		set_SPropValue_proptag (&(recipient->in.req_lpProps[0]), PR_SEND_INTERNET_ENCODING, (gconstpointer )&val);
 
 		val = RECIP_SENDABLE | RECIP_ORGANIZER;
-		set_SPropValue_proptag (&(recipient->in.req_lpProps[1]), PR_RECIPIENTS_FLAGS, (gconstpointer )&val);
+		set_SPropValue_proptag (&(recipient->in.req_lpProps[1]), PR_RECIPIENT_FLAGS, (gconstpointer )&val);
 
 		val = olResponseNone;
 		set_SPropValue_proptag (&(recipient->in.req_lpProps[2]), PR_RECIPIENT_TRACKSTATUS, (gconstpointer )&val);
@@ -384,7 +384,7 @@ exchange_mapi_cal_util_fetch_recipients (ECalComponent *comp, GSList **recip_lis
 		set_SPropValue_proptag (&(recipient->in.req_lpProps[0]), PR_SEND_INTERNET_ENCODING, (gconstpointer )&val);
 
 		val = RECIP_SENDABLE | (!g_ascii_strcasecmp(str, org) ? RECIP_ORGANIZER : 0);
-		set_SPropValue_proptag (&(recipient->in.req_lpProps[1]), PR_RECIPIENTS_FLAGS, (gconstpointer )&val);
+		set_SPropValue_proptag (&(recipient->in.req_lpProps[1]), PR_RECIPIENT_FLAGS, (gconstpointer )&val);
 
 		param = icalproperty_get_first_parameter (att_prop, ICAL_PARTSTAT_PARAMETER);
 		val = get_trackstatus_from_partstat (icalparameter_get_partstat(param));
@@ -493,7 +493,7 @@ ical_attendees_from_props (icalcomponent *ical_comp, GSList *recipients, gboolea
 		else
 			continue;
 
-		flags = (const uint32_t *) get_SPropValue_SRow_data (&recip->out_SRow, PR_RECIPIENTS_FLAGS);
+		flags = (const uint32_t *) get_SPropValue_SRow_data (&recip->out_SRow, PR_RECIPIENT_FLAGS);
 
 		if (flags && (*flags & RECIP_ORGANIZER)) {
 			prop = icalproperty_new_organizer (val);
@@ -528,11 +528,14 @@ ical_attendees_from_props (icalcomponent *ical_comp, GSList *recipients, gboolea
 			ui32 = (const uint32_t *) get_SPropValue_SRow_data (&recip->out_SRow, PR_RECIPIENT_TYPE);
 			param = icalparameter_new_role (get_role_from_type (ui32 ? *ui32 : olTo));
 			icalproperty_add_parameter (prop, param);
-#if 0
+
 			/* CALENDAR USER TYPE */
-			param = icalparameter_new_cutype ();
+			param = NULL;
+			if (ui32 && *ui32 == 0x03)
+				param = icalparameter_new_cutype (ICAL_CUTYPE_RESOURCE);
+			if (!param)
+				param = icalparameter_new_cutype (ICAL_CUTYPE_INDIVIDUAL);
 			icalproperty_add_parameter (prop, param);
-#endif
 		}
 
 		if (prop)
@@ -816,7 +819,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 			b = (const bool *)find_mapi_SPropValue_data(properties, PR_RESPONSE_REQUESTED);
 			ical_attendees_from_props (ical_comp, recipients, (b && *b));
 			if (icalcomponent_get_first_property (ical_comp, ICAL_ORGANIZER_PROPERTY) == NULL) {
-				gchar *val;
+				gchar *val, *sender_free = NULL, *sent_free = NULL;
 //				const gchar *sender_name = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENDER_NAME_UNICODE);
 				const gchar *sender_email_type = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENDER_ADDRTYPE_UNICODE);
 				const gchar *sender_email = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENDER_EMAIL_ADDRESS_UNICODE);
@@ -824,10 +827,14 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 				const gchar *sent_email_type = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_ADDRTYPE_UNICODE);
 				const gchar *sent_email = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE);
 
-				if (!g_utf8_collate (sender_email_type, "EX"))
-					sender_email = exchange_mapi_connection_ex_to_smtp (conn, sender_email);
-				if (!g_utf8_collate (sent_email_type, "EX"))
-					sent_email = exchange_mapi_connection_ex_to_smtp (conn, sent_email);
+				if (!g_utf8_collate (sender_email_type, "EX")) {
+					sender_free = exchange_mapi_connection_ex_to_smtp (conn, sender_email, NULL);
+					sender_email = sender_free;
+				}
+				if (!g_utf8_collate (sent_email_type, "EX")) {
+					sent_free = exchange_mapi_connection_ex_to_smtp (conn, sent_email, NULL);
+					sent_email = sent_free;
+				}
 
 				val = g_strdup_printf ("MAILTO:%s", sent_email);
 				prop = icalproperty_new_organizer (val);
@@ -844,6 +851,9 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 				}
 
 				icalcomponent_add_property (ical_comp, prop);
+
+				g_free (sender_free);
+				g_free (sent_free);
 			}
 		}
 
@@ -957,8 +967,6 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 	return comp;
 }
 
-#define TEMP_ATTACH_STORE ".evolution/cache/tmp"
-
 struct fetch_camel_cal_data {
 	icalcomponent_kind kind;
 	icalproperty_method method;
@@ -972,13 +980,12 @@ fetch_camel_cal_comp_cb (FetchItemsCallbackData *item_data, gpointer data)
 	ECalComponent *comp = NULL;
 	mapi_id_t mid = 0;
 	icalcomponent *icalcomp = NULL;
-	gchar *str = NULL, *smid = NULL, *filename, *fileuri;
+	gchar *str = NULL, *smid = NULL, *fileuri;
 
 	g_return_val_if_fail (item_data != NULL, FALSE);
 	g_return_val_if_fail (fccd != NULL, FALSE);
 
-	filename = g_build_filename (g_get_home_dir (), TEMP_ATTACH_STORE, NULL);
-	fileuri = g_filename_to_uri (filename, NULL, NULL);
+	fileuri = g_filename_to_uri (g_get_tmp_dir (), NULL, NULL);
 
 	if (!comp) {
 		/* read component from a mail, if not found in the calendar */
@@ -997,7 +1004,6 @@ fetch_camel_cal_comp_cb (FetchItemsCallbackData *item_data, gpointer data)
 	}
 
 	g_free (fileuri);
-	g_free (filename);
 
 	icalcomp = e_cal_util_new_top_level ();
 	icalcomponent_set_method (icalcomp, fccd->method);
@@ -1046,12 +1052,12 @@ exchange_mapi_cal_util_camel_helper (ExchangeMapiConnection *conn, mapi_id_t ori
 		exchange_mapi_connection_fetch_object_props (conn, NULL, orig_fid, orig_mid, obj_message,
 					exchange_mapi_cal_utils_get_props_cb, GINT_TO_POINTER (fccd.kind),
 					fetch_camel_cal_comp_cb, &fccd,
-					MAPI_OPTIONS_FETCH_ALL);
+					MAPI_OPTIONS_FETCH_ALL, NULL);
 	else
 		exchange_mapi_connection_fetch_item (conn, orig_fid, orig_mid,
 					exchange_mapi_cal_utils_get_props_cb, GINT_TO_POINTER (fccd.kind),
 					fetch_camel_cal_comp_cb, &fccd,
-					MAPI_OPTIONS_FETCH_ALL);
+					MAPI_OPTIONS_FETCH_ALL, NULL);
 
 	return fccd.result_data;
 }
@@ -1075,7 +1081,7 @@ exchange_mapi_cal_utils_add_named_ids (ExchangeMapiConnection *conn, mapi_id_t f
 	icalcomponent_kind kind = pkind;
 
 	if (!props) {
-		if (!exchange_mapi_connection_resolve_named_props (conn, fid, common_nids, G_N_ELEMENTS (common_nids)))
+		if (!exchange_mapi_connection_resolve_named_props (conn, fid, common_nids, G_N_ELEMENTS (common_nids), NULL))
 			return FALSE;
 	} else if (!exchange_mapi_utils_add_named_ids_to_props_array (conn, fid, mem_ctx, props, common_nids, G_N_ELEMENTS (common_nids)))
 		return FALSE;
@@ -1129,7 +1135,7 @@ appt_build_name_id (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem
 	};
 
 	if (!props)
-		return exchange_mapi_connection_resolve_named_props (conn, fid, nids, G_N_ELEMENTS (nids));
+		return exchange_mapi_connection_resolve_named_props (conn, fid, nids, G_N_ELEMENTS (nids), NULL);
 
 	return exchange_mapi_utils_add_named_ids_to_props_array (conn, fid, mem_ctx, props, nids, G_N_ELEMENTS (nids));
 }
@@ -1158,7 +1164,7 @@ task_build_name_id (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem
 	};
 
 	if (!props)
-		return exchange_mapi_connection_resolve_named_props (conn, fid, nids, G_N_ELEMENTS (nids));
+		return exchange_mapi_connection_resolve_named_props (conn, fid, nids, G_N_ELEMENTS (nids), NULL);
 
 	return exchange_mapi_utils_add_named_ids_to_props_array (conn, fid, mem_ctx, props, nids, G_N_ELEMENTS (nids));
 }
@@ -1174,7 +1180,7 @@ note_build_name_id (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem
 	};
 
 	if (!props)
-		return exchange_mapi_connection_resolve_named_props (conn, fid, nids, G_N_ELEMENTS (nids));
+		return exchange_mapi_connection_resolve_named_props (conn, fid, nids, G_N_ELEMENTS (nids), NULL);
 
 	return exchange_mapi_utils_add_named_ids_to_props_array (conn, fid, mem_ctx, props, nids, G_N_ELEMENTS (nids));
 }
@@ -1259,9 +1265,9 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 			return FALSE;	\
 		} G_STMT_END
 
-	#define set_datetime_value(hex, dtval) G_STMT_START { 		\
-		struct FILETIME	filetime; 				\
-		NTTIME		nttime; 					\
+	#define set_datetime_value(hex, dtval) G_STMT_START {		\
+		struct FILETIME	filetime;				\
+		NTTIME		nttime;					\
 									\
 		nttime = timeval_to_nttime(dtval);				\
 									\
@@ -1273,8 +1279,8 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 		} G_STMT_END
 
 	#define set_named_datetime_value(named_id, dtval) G_STMT_START { \
-		struct FILETIME	filetime; 				\
-		NTTIME		nttime; 					\
+		struct FILETIME	filetime;				\
+		NTTIME		nttime;					\
 									\
 		nttime = timeval_to_nttime(dtval);				\
 									\
@@ -1780,15 +1786,23 @@ exchange_mapi_cal_util_get_new_appt_id (ExchangeMapiConnection *conn, mapi_id_t
 		id = g_random_int ();
 		if (id) {
 			GSList *ids = NULL;
+			TALLOC_CTX *mem_ctx = talloc_init ("ExchangeMAPI_get_new_appt_id");
+
 			set_SPropValue_proptag (&sprop, PR_OWNER_APPT_ID, (gconstpointer ) &id);
-			cast_mapi_SPropValue (&(res.res.resProperty.lpProp), &sprop);
-			ids = exchange_mapi_connection_check_restriction (conn, fid, 0, &res);
+			cast_mapi_SPropValue (
+				#ifdef HAVE_MEMCTX_ON_CAST_MAPI_SPROPVALUE
+				mem_ctx,
+				#endif
+				&(res.res.resProperty.lpProp), &sprop);
+			ids = exchange_mapi_connection_check_restriction (conn, fid, 0, &res, NULL);
 			if (ids) {
 				GSList *l;
 				for (l = ids; l; l = l->next)
 					g_free (l->data);
 			} else
 				found = TRUE;
+
+			talloc_free (mem_ctx);
 		}
 	};
 
@@ -1841,9 +1855,9 @@ populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GLi
 	uint16_t	event_start;
 	uint16_t	event_end;
 	uint32_t	i;
-	uint32_t       	hour;
+	uint32_t	hour;
 	uint32_t	day;
-	const char	*month_name;
+	const gchar	*month_name;
 	uint32_t	minutes;
 	uint32_t	real_month;
 	gchar *date_string = NULL;
@@ -1875,7 +1889,7 @@ populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GLi
 				day = ((event_start - (60 * hour)) / 1440) + 1;
 				minutes = (event_start - (60 * hour)) % 1440;
 				real_month = month - (year * 16);
-				
+
 				date_string = g_strdup_printf ("%.2u-%.2u-%.2u", year, real_month, day);
 				start = g_strdup_printf ("%sT%.2u:%.2u:00Z", date_string, hour + daylight, minutes);
 				g_free (date_string);
@@ -1896,13 +1910,13 @@ populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GLi
 				end_date = mapi_get_date_from_string (end);
 
 				memset (&ipt, 0, sizeof (struct icalperiodtype));
-				
+
 				itt = icaltime_from_timet_with_zone (start_date, 0, icaltimezone_get_utc_timezone ());
 				ipt.start = itt;
 
 				itt = icaltime_from_timet_with_zone (end_date, 0, icaltimezone_get_utc_timezone ());
 				ipt.end = itt;
-			
+
 				icalcomp = e_cal_component_get_icalcomponent (comp);
 				icalprop = icalproperty_new_freebusy (ipt);
 
@@ -1922,25 +1936,24 @@ populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GLi
 }
 
 gboolean
-exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, GList *users, time_t start, time_t end, GList **freebusy)
+exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, const GList *users, time_t start, time_t end, GList **freebusy, GError **mapi_error)
 {
 	struct SRow		aRow;
-	enum MAPISTATUS		retval;
+	enum MAPISTATUS		ms;
 	uint32_t		i;
 	mapi_object_t           obj_store;
-	GList *l;
+	const GList *l;
 
 	const uint32_t			*publish_start;
-	const struct LongArray_r       	*busy_months;
+	const struct LongArray_r	*busy_months;
 	const struct BinaryArray_r	*busy_events;
-	const struct LongArray_r       	*tentative_months;
+	const struct LongArray_r	*tentative_months;
 	const struct BinaryArray_r	*tentative_events;
-	const struct LongArray_r       	*oof_months;
+	const struct LongArray_r	*oof_months;
 	const struct BinaryArray_r	*oof_events;
 	uint32_t			year;
 	uint32_t			event_year;
 
-	gchar *name = NULL;
 	ECalComponent *comp;
 	ECalComponentAttendee attendee;
 	GSList *attendee_list = NULL;
@@ -1948,12 +1961,22 @@ exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, GList
 	icaltimetype start_time, end_time;
 	icaltimezone *default_zone = NULL;
 
-	exchange_mapi_connection_get_public_folder (conn, &obj_store);
-	
-	for ( l = users; l != NULL; l = g_list_next (l)) {
-		retval = GetUserFreeBusyData (&obj_store, (const gchar *)l->data, &aRow);
+	if (!exchange_mapi_connection_get_public_folder (conn, &obj_store, mapi_error)) {
+		return FALSE;
+	}
 
-		if (retval != MAPI_E_SUCCESS) return false;
+	for ( l = users; l != NULL; l = g_list_next (l)) {
+		ms = GetUserFreeBusyData (&obj_store, (const gchar *)l->data, &aRow);
+
+		if (ms != MAPI_E_SUCCESS) {
+			gchar *context = g_strconcat ("GetUserFreeBusyData for ", l->data, NULL);
+
+			make_mapi_error (mapi_error, context, ms);
+
+			g_free (context);
+
+			return FALSE;
+		}
 
 		/* Step 2. Dump properties */
 		publish_start = (const uint32_t *) find_SPropValue_data(&aRow, PR_FREEBUSY_START_RANGE);
@@ -1977,8 +2000,6 @@ exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, GList
 		icalcomponent_set_dtend (icalcomp, end_time);
 
 		memset (&attendee, 0, sizeof (ECalComponentAttendee));
-		if (name)
-			attendee.cn = name;
 		if (l->data)
 			attendee.value = l->data;
 
@@ -1990,8 +2011,6 @@ exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, GList
 
 		e_cal_component_set_attendee_list (comp, attendee_list);
 		g_slist_free (attendee_list);
-		g_free ((gchar *) name);
-		g_free ((gchar *) l->data);
 
 		if (busy_months && ((*(const uint32_t *) busy_months) != MAPI_E_NOT_FOUND) &&
 		    busy_events && ((*(const uint32_t *) busy_events) != MAPI_E_NOT_FOUND)) {
@@ -2000,7 +2019,7 @@ exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, GList
 				populate_freebusy_data (&busy_events->lpbin[i], busy_months->lpl[i], event_year, freebusy, "Busy", comp);
 			}
 		}
-		
+
 		if (tentative_months && ((*(const uint32_t *) tentative_months) != MAPI_E_NOT_FOUND) &&
 		    tentative_events && ((*(const uint32_t *) tentative_events) != MAPI_E_NOT_FOUND)) {
 			for (i = 0; i < tentative_months->cValues; i++) {
@@ -2019,9 +2038,10 @@ exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, GList
 
 		e_cal_component_commit_sequence (comp);
 		*freebusy = g_list_append (*freebusy, e_cal_component_get_as_string (comp));
-//		g_object_unref (comp);
+		g_object_unref (comp);
 		MAPIFreeBuffer(aRow.lpProps);
 	}
+
 	return TRUE;
 }
 
